@@ -20,22 +20,35 @@ from tinynetrc import Netrc
 
 from _packagedcode.models import DependentPackage
 from _packagedcode.pypi import PipRequirementsFileHandler
+from _packagedcode.pypi import PythonSetupPyHandler
 from _packagedcode.pypi import can_process_dependent_package
 from python_inspector import dependencies
 from python_inspector import utils
 from python_inspector import utils_pypi
 from python_inspector.cli_utils import FileOptionType
+from python_inspector.package_data import get_pypi_data_from_purl
+from python_inspector.resolution import contain_string
+from python_inspector.resolution import get_deps_from_distribution
 from python_inspector.resolution import get_environment_marker_from_environment
+from python_inspector.resolution import get_python_version_from_env_tag
 from python_inspector.resolution import get_resolved_dependencies
+from python_inspector.resolution import parse_deps_from_setup_py_insecurely
 
 TRACE = False
 
-__version__ = "0.6.3"
+__version__ = "0.7.1"
 
 DEFAULT_PYTHON_VERSION = "38"
 PYPI_SIMPLE_URL = "https://pypi.org/simple"
 
 
+def print_version(ctx, param, value):
+    if not value or ctx.resilient_parsing:
+        return
+    click.echo(f"Python-inspector version: {__version__}")
+    ctx.exit()
+
+
 @click.command()
 @click.pass_context
 @click.option(
@@ -50,13 +63,14 @@ PYPI_SIMPLE_URL = "https://pypi.org/simple"
     "This option can be used multiple times.",
 )
 @click.option(
-    "-n",
-    "--netrc",
-    "netrc_file",
+    "-s",
+    "--setup-py",
+    "setup_py_file",
     type=click.Path(exists=True, readable=True, path_type=str, dir_okay=False),
-    metavar="NETRC-FILE",
+    metavar="SETUP-PY-FILE",
     required=False,
-    help="Netrc file to use for authentication. ",
+    help="Path to setuptools setup.py file listing dependencies and metadata. "
+    "This option can be used multiple times.",
 )
 @click.option(
     "--spec",
@@ -117,9 +131,20 @@ PYPI_SIMPLE_URL = "https://pypi.org/simple"
     help="Write output as pretty-printed JSON to FILE as a tree in the style of pipdeptree. "
     "Use the special '-' file name to print results on screen/stdout.",
 )
+@click.option(
+    "-n",
+    "--netrc",
+    "netrc_file",
+    type=click.Path(exists=True, readable=True, path_type=str, dir_okay=False),
+    metavar="NETRC-FILE",
+    hidden=True,
+    required=False,
+    help="Netrc file to use for authentication. ",
+)
 @click.option(
     "--max-rounds",
     "max_rounds",
+    hidden=True,
     type=int,
     default=200000,
     help="Increase the max rounds whenever the resolution is too deep",
@@ -136,31 +161,49 @@ PYPI_SIMPLE_URL = "https://pypi.org/simple"
     help="Use PyPI JSON API to fetch dependency data. Faster but not always correct. "
     "--index-url are ignored when this option is active.",
 )
+@click.option(
+    "--analyze-setup-py-insecurely",
+    is_flag=True,
+    help="Enable collection of requirements in setup.py that compute these"
+    " dynamically. This is an insecure operation as it can run arbitrary code.",
+)
 @click.option(
     "--verbose",
     is_flag=True,
     hidden=True,
     help="Enable debug output.",
 )
+@click.option(
+    "-V",
+    "--version",
+    is_flag=True,
+    is_eager=True,
+    expose_value=False,
+    callback=print_version,
+    help="Show the version and exit.",
+)
 @click.help_option("-h", "--help")
 def resolve_dependencies(
     ctx,
     requirement_files,
-    netrc_file,
+    setup_py_file,
     specifiers,
     python_version,
     operating_system,
     index_urls,
     json_output,
     pdt_output,
+    netrc_file,
     max_rounds,
     use_cached_index=False,
     use_pypi_json_api=False,
+    analyze_setup_py_insecurely=False,
     verbose=TRACE,
 ):
     """
-    Resolve the dependencies of the packages listed in REQUIREMENT-FILE(s) file
-    and SPECIFIER(s) and save the results as JSON to FILE.
+    Resolve the dependencies for the package requirements listed in one or
+    more REQUIREMENT-FILE file, one or more SPECIFIER and one setuptools
+    SETUP-PY-FILE file and save the results as JSON to FILE.
 
     Resolve the dependencies for the requested ``--python-version`` PYVER and
     ``--operating_system`` OS combination defaulting Python version 3.8 and
@@ -221,6 +264,56 @@ def resolve_dependencies(
         dep = dependencies.get_dependency(specifier=specifier)
         direct_dependencies.append(dep)
 
+    if setup_py_file:
+        package_data = list(PythonSetupPyHandler.parse(location=setup_py_file))
+        assert len(package_data) == 1
+        package_data = package_data[0]
+        # validate if python require matches our current python version
+        python_requires = package_data.extra_data.get("python_requires")
+        if not utils_pypi.valid_python_version(
+            python_version=get_python_version_from_env_tag(python_version),
+            python_requires=python_requires,
+        ):
+            click.secho(
+                f"Python version {get_python_version_from_env_tag(python_version)} "
+                f"is not compatible with setup.py {setup_py_file} "
+                f"python_requires {python_requires}",
+                err=True,
+            )
+            ctx.exit(1)
+
+        for dep in package_data.dependencies:
+            # TODO : we need to handle to all the scopes
+            if dep.scope == "install":
+                direct_dependencies.append(dep)
+
+        if not package_data.dependencies:
+            has_deps = False
+            if contain_string(string="requirements.txt", files=[setup_py_file]):
+                # Look in requirements file if and only if thy are refered in setup.py or setup.cfg
+                # And no deps have been yielded by requirements file.
+
+                location = os.path.dirname(setup_py_file)
+                requirement_location = os.path.join(
+                    location,
+                    "requirements.txt",
+                )
+                deps = get_deps_from_distribution(
+                    handler=PipRequirementsFileHandler,
+                    location=requirement_location,
+                )
+                if deps:
+                    has_deps = True
+                    direct_dependencies.extend(deps)
+
+            if not has_deps and contain_string(string="_require", files=[setup_py_file]):
+                if analyze_setup_py_insecurely:
+                    direct_dependencies.extend(
+                        parse_deps_from_setup_py_insecurely(setup_py=setup_py_file)
+                    )
+                else:
+                    raise Exception("Unable to collect setup.py dependencies securely")
+
     if not direct_dependencies:
         click.secho("Error: no requirements requested.")
         ctx.exit(1)
@@ -267,7 +360,7 @@ def resolve_dependencies(
             click.secho(f" {repo}")
 
     # resolve dependencies proper
-    requirements, resolved_dependencies = resolve(
+    requirements, resolved_dependencies, purls = resolve(
         direct_dependencies=direct_dependencies,
         environment=environment,
         repos=repos,
@@ -275,6 +368,7 @@ def resolve_dependencies(
         max_rounds=max_rounds,
         verbose=verbose,
         pdt_output=pdt_output,
+        analyze_setup_py_insecurely=analyze_setup_py_insecurely,
     )
 
     cli_options = [f"--requirement {rf}" for rf in requirement_files]
@@ -300,12 +394,20 @@ def resolve_dependencies(
         errors=[],
     )
 
+    packages = []
+
+    for package in purls:
+        packages.extend(
+            list(get_pypi_data_from_purl(package, repos=repos, environment=environment)),
+        )
+
     if json_output:
         write_output(
             headers=headers,
             requirements=requirements,
             resolved_dependencies=resolved_dependencies,
             json_output=json_output,
+            packages=packages,
         )
 
     else:
@@ -314,6 +416,7 @@ def resolve_dependencies(
             requirements=requirements,
             resolved_dependencies=resolved_dependencies,
             json_output=pdt_output,
+            packages=packages,
             pdt_output=True,
         )
 
@@ -329,6 +432,7 @@ def resolve(
     max_rounds=200000,
     verbose=False,
     pdt_output=False,
+    analyze_setup_py_insecurely=False,
 ):
     """
     Resolve dependencies given a ``direct_dependencies`` list of
@@ -346,7 +450,7 @@ def resolve(
         )
     )
 
-    resolved_dependencies = get_resolved_dependencies(
+    resolved_dependencies, packages = get_resolved_dependencies(
         requirements=requirements,
         environment=environment,
         repos=repos,
@@ -354,11 +458,12 @@ def resolve(
         max_rounds=max_rounds,
         verbose=verbose,
         pdt_output=pdt_output,
+        analyze_setup_py_insecurely=analyze_setup_py_insecurely,
     )
 
     initial_requirements = [d.to_dict() for d in direct_dependencies]
 
-    return initial_requirements, resolved_dependencies
+    return initial_requirements, resolved_dependencies, packages
 
 
 def get_requirements_from_direct_dependencies(
@@ -378,19 +483,26 @@ def get_requirements_from_direct_dependencies(
                 yield req
 
 
-def write_output(headers, requirements, resolved_dependencies, json_output, pdt_output=False):
+def write_output(
+    headers, requirements, resolved_dependencies, json_output, packages, pdt_output=False
+):
     """
     Write headers, requirements and resolved_dependencies as JSON to ``json_output``.
     Return the output data.
     """
+
     if not pdt_output:
         output = dict(
             headers=headers,
             requirements=requirements,
             resolved_dependencies=resolved_dependencies,
+            packages=packages,
         )
     else:
-        output = resolved_dependencies
+        output = dict(
+            resolved_dependencies=resolved_dependencies,
+            packages=packages,
+        )
 
     json.dump(output, json_output, indent=2)
     return output
